 /* IE 6-8 fallback colors */
.marker-cluster-small {
	background-color: rgb(181, 226, 140);
	}
.marker-cluster-small div {
	background-color: rgb(110, 204, 57);
	}

.marker-cluster-medium {
	background-color: rgb(241, 211, 87);
	}
.marker-cluster-medium div {
	background-color: rgb(240, 194, 12);
	}

.marker-cluster-large {
	background-color: rgb(253, 156, 115);
	}
.marker-cluster-large div {
	background-color: rgb(241, 128, 23);
}

