/*
 * Nodeshot typography
 * keep here all instructions related to fonts and text
 */

@font-face {
    font-family: 'Montserrat';
    font-style: normal;
    font-weight: 400;
    src: local('Montserrat-Regular'), url(../fonts/montserrat-400.woff2) format('woff2'), url(../fonts/montserrat-400.woff) format('woff');
}
@font-face {
    font-family: 'Montserrat';
    font-style: normal;
    font-weight: 700;
    src: local('Montserrat-Bold'), url(../fonts/montserrat-700.woff2) format('woff2'), url(../fonts/montserrat-700.woff) format('woff');
}

body,
.form-control
{
    font-size: 14px;
    font-family: 'Montserrat', sans-serif;
}

input, textarea, button{
    font-size: 14px;
    font-family: 'Montserrat', sans-serif;
}

#top-bar{
    font-size: 15px;
}

#top-bar .notifications .bg{
    font: bold 14px "Arial", sans-serif;
}

#top-bar ul li a{ text-decoration: none }
#ns-top-nav-links a span{ font-size: 15px }

article{
    text-align: justify;
}

article.center-stage > *:first-child,
.content > *:first-child{ margin-top: 0 }

body article h1{ letter-spacing: -1.25px }
body article h2{ letter-spacing: -1px }
h1,
h2,
h3,
h5,
h5,
h6{ text-align: left }

h1.title,
h2.title{ margin-top: 0 }

.row-ns h1.title,
.row-ns h2.title{ margin-bottom: 0 }

body article blockquote{
    padding-top: 15px;
    padding-bottom: 15px;
    font-size: 14px;
}

.loading{ font-size: 15px }

#map-legend{
    font-family: Arial;
    font-size: 12px;
}

#map-legend h5{
    text-transform: uppercase;
    font-variant: small-caps;
    font-family: Arial;
    font-size: 10px;
    font-weight: bold;
}

.user-dialog .form-control{
    font-size: 13px
}

.user-dialog p.text{
    font: normal 14px/20px "Arial", sans-serif;
}

.side-panel h1,
.side-panel h2,
.side-panel h3{
    margin: 0 0 13px;
    font: bold 14px "Arial", sans-serif;
    font-variant: small-caps;
    text-transform: uppercase;
}

.side-panel{
    font: normal 14px "Helvetica", "Arial", sans-serif;
}

/* --- LARGE SCREENS --- */

@media screen and (min-width: 1150px){
    .multicolumn-md{
        -moz-column-count: 2;
        -moz-column-gap: 32px;
        -webkit-column-count: 2;
        -webkit-column-gap: 32px;
    }
}
