/* ==========================================================================
   Author's custom styles
   ========================================================================== */
body {
    background: #D6D4C9;
    background: -moz-linear-gradient(#fff, #999);
    background: -webkit-linear-gradient(#fff, #999);
    background: -o-linear-gradient(#fff, #999);
}

.row {
    margin-top: 25px;
}

.strong {
   font-weight: bold;
}

#loading {
   background-color: #CCCCCC;
   height: 50px;
   width: 150px;
   position: fixed;
   z-index: 100000;
   left: 50%;
   top: 50%;
   margin: -25px 0 0 -25px;
   border-radius: 4px;
   border: thin solid;
   border-color: #000000;
   padding:5px;
}

.mapLegend {
    padding: 6px 8px;
    font: 14px/16px Arial, Helvetica, sans-serif;
    background: white;
    background: rgba(255, 255, 255, 0.8);
    box-shadow: 0 0 15px rgba(0, 0, 0, 0.2);
    border-radius: 5px;
    text-align: left;
}

.circle {
    border-radius: 50%;
    width: 12px;
    height: 12px;
    /* width and height can be anything, as long as they're equal */
}

.overlay {
    background:transparent url(images/overlay.png) repeat top left;
    position:fixed;
    top:0px;
    bottom:0px;
    left:0px;
    right:0px;
    height: 100%;
    width: 100%;
    z-index:10000;
}

#serviceRequestForm {
    width: 450px;
    height: 700px;
    line-height: 1.2;
    position: fixed;
    top: 30%;
    left: 50%;
    margin-top: -100px;
    margin-left: -250px;
    background: #D6D4C9;
    border-radius: 5px;
    border-color: black;
    border: thin solid;
    text-align: center;
    padding-left: 20px;
    padding-top:20px;
    padding-right: 20px;
    z-index: 10001;
    /* 1px higher than the overlay layer */
}

a.boxclose {
    float:right;
    width:26px;
    height:26px;
    background:transparent url(images/cancel.png) repeat top left;
    margin-top:-30px;
    margin-right:-30px;
    cursor:pointer;
}

.thumbnail {
    width:200px;
    height: 150px;
    padding: 0px;
}

.remove_marker {
    background-color: #ff0000;
    color: #000000
}

.confirm_marker {
    background-color: #00ff00;
    color: #000000
}

#nodelist {
    background-color: #c0c0c0;
    color: #000000;
    font-size: 14px;
    max-height: 500px;
    padding: 2px;
    border-radius: 4px;
    overflow:auto;
}

#nodelistContainer {
    background-color: #c0c0c0;
    color: #000000;
    font-size: 16px;
    text-align: center;
        border-radius: 10px 10px 0px 0px;
   border: thin solid;
    border-color: grey;
}

#mapContainer {
    background-color: #c0c0c0;
    color: #000000;
    font-size: 16px;
   border-radius: 10px 10px 0px 0px;
   border: thin solid;
    border-color: grey;
}


.error_messages {
    font-size: 13px;
    margin:1px;
    color:red;
}

.thanks_messages {
    font-size: 13px;
    margin:1px;
    color:blue;
}

#requestDetails {
    width: 960px;
    height: 700px;
    line-height: 20px;
    position: fixed;
    top: 30%;
    left: 50%;
    margin-top: -100px;
    margin-left: -480px;
    background: #D6D4C9;
    border-radius: 5px;
    border-color: black;
    border: thin solid;
    /*text-align: center;*/
    padding-left: 20px;
    padding-top:20px;
    padding-right: 20px;
    z-index: 10001;
    /* 1px higher than the overlay layer */
}

.intest {
    background-color: #33AB67;
    color: #FFFFFF;
    font-size: 16px;
    padding: 2px;
    text-align: center;
    font-weight: bold;
    border-radius: 10px 10px 0px 0px;
    /*border: thin solid;*/
    /*border-color: black;*/
    
}

.requestSubBox {

    background-color: #FFFFFF;
    margin-bottom: 10px;
    border-radius: 10px 10px 0px 0px;
    border: thin solid;
    border-color: grey;
}

#votes img {
    width: 24px;
    cursor: pointer;
}

#post_comments {
       margin:5px;
}

#comment_list {
    background-color: #c0c0c0;
    color: #000000;
    font-size: 12px;
    margin: 1px;
    padding: 5px;
    max-height: 200px;
    overflow:auto;
}

.your_comment {
    font-size:14px;
    color:#000000;
    background-color: #FFFFFF;
    word-wrap: break-word;
    border-style:solid;
    border-width:1px;
    padding: 3px;
    margin:5px;
    width: 80%;
    height: 100px;
}

.comment_text {
    font-size:14px;
    color:#000000;
    background-color: #FFFFFF;
    word-wrap: break-word;
    border-style:solid;
    border-width:1px;
    border-radius: 3px;
    border-color: grey;
    padding: 3px;
    width: 80%;
}

.comment_user {
    font-size:12px;
    color:#0000FF;
    word-wrap: break-word;
    padding: 1px;

}

.comment_date {
    font-size:11px;
    color:#000000;
    word-wrap: break-word;
    padding: 1px;

}

.userLogged {
    color: #ffffff;
}

.user {
    font-size:10px;
    color:#FFFFFF;
    background-color: #CCCCCC;
    word-wrap: break-word;
}