{% load i18n %}
<h1>{% if token_fail %}{% trans "Bad Token" %}{% else %}{% trans "Change Password" %}{% endif %}</h1>

{% if token_fail %}
    {% url "acct_passwd_reset" as passwd_reset_url %}
    <p>{% blocktrans %}The password reset link was invalid, possibly because it has already been used.{% endblocktrans %}</p>
{% else %}
    {% if form %}
        <form method="POST" action="">
            {% csrf_token %}
            <fieldset>
                {{ form }}
            </fieldset>
            <div class="actions">
                 <button type="submit" class="btn primary">{% trans "change password" %}"</button>
            </div>
        </form>
    {% else %}
        <p>{% trans "You successfully changed your password!" %}</p>
    {% endif %}
{% endif %}
