/*
 * Nodeshot colors
 * keep here all instructions related to colors,
 * background colors, borders, ecc
 */

body{
    background-color: #f2f2f2;
    /* body text */
    color: #303030;
}

/* --- LOGO --- */
#top-bar h1 a{
    background: transparent url(../images/nodeshot-logo.svg) no-repeat scroll 0 50%
}

a:focus{ outline: 0 none }

#top-bar a:focus{ outline: 1px dotted rgba(255, 255, 255, 0.7) }

#top-bar{
    background-color: #de5c44;
    background: -moz-linear-gradient(left, #df5d43 0%, #f0775f 50%, #df5d43 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, right top, color-stop(0%,#df5d43), color-stop(50%,#f0775f), color-stop(100%,#df5d43)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(left, #df5d43 0%,#f0775f 50%,#df5d43 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(left, #df5d43 0%,#f0775f 50%,#df5d43 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(left, #df5d43 0%,#f0775f 50%,#df5d43 100%); /* IE10+ */
    background: linear-gradient(to right, #df5d43 0%,#f0775f 50%,#df5d43 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#df5d43', endColorstr='#df5d43',GradientType=1 ); /* IE6-9 */
}

#nav-bar, .breadcrumb{
    background: #383838; /* Old browsers */
    background: -moz-linear-gradient(top,  #383838 0%, #4d4d4d 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#383838), color-stop(100%,#4d4d4d)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #383838 0%,#4d4d4d 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #383838 0%,#4d4d4d 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #383838 0%,#4d4d4d 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #383838 0%,#4d4d4d 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#383838', endColorstr='#4d4d4d',GradientType=0 ); /* IE6-9 */
}


#notifications{
    background-color: #f2f2f2;
    border: 1px solid #d9dada;
    border-top: 0 none;
    -webkit-box-shadow: 0 6px 12px rgba(0,0,0,.175);
    box-shadow: 0 6px 12px rgba(0,0,0,.175);
    color: #32363a;
}

#notifications .arrow{
    border-left: 8px solid transparent;
    border-right: 8px solid transparent;
    border-bottom: 12px solid #eaeaea;
}

#notifications a{
    color: #32363a !important;
    text-decoration: underline
}

#notifications a:hover{
    text-decoration: none;
}

#notifications .header{
    border-bottom: 1px solid #d9dade;
    background-color: #eaeaea;
}

#notifications li{
    border-top: 1px solid #fff;
    border-bottom: 1px solid #d9dade;
    color: #696a6e;
}
#notifications li.unread{
    background-color: #eaeaea;
}
#notifications li:hover{
    background-color: #e2e6ed;
}

#notifications li span{
    color: #242429;
}
#notifications .time{
    color: #a6a6ab
}

.breadcrumb{ border-radius: 0 }

body > header{
    color: #fff;
}

#general-search-input::-webkit-input-placeholder {
    color: #979696;
}
#general-search-input:-moz-placeholder {
    /* FF 4-18 */
    color: #979696;
}
#general-search-input::-moz-placeholder {
    /* FF 19+ */
    color: #979696;
}
#general-search-input:-ms-input-placeholder {
    /* IE 10+ */
    color: #979696;
}

#top-bar a, .breadcrumb a, header a, header a:hover{ color: #fff }
/*.breadcrumb a{ text-decoration: underline }*/

#main-actions > ul > li > a{
    border-left: 1px solid rgba(255, 255, 255, 0);
    border-right: 1px solid rgba(255, 255, 255, 0);
    margin-left: -1px;
}

#main-actions > ul > li > a:hover,
#main-actions > ul > li > a:focus,
#main-actions li.open a{
    background-color: #df5134;
    border-left: 1px solid rgba(255, 255, 255, 0.3);
    border-right: 1px solid rgba(255, 255, 255, 0.3);
    outline: 0 none;
}
#main-actions > ul > li > :active { background-color: #ff866d }

#top-bar li ul{
    padding: 0;
    margin: -2px 0 0 0;
    border: 1px solid rgba(255, 255, 255, 0.3) !important;
}

#top-bar li li a{
    margin: 0;
    background-color: #df5134
}

#top-bar li li a:hover{
    background-color: #ff866d
}

#top-bar .notifications .bg{
    background-color: #4d4d4d;
}

#ns-top-nav-links > ul > li > a{ color: #dbdbdb }
#ns-top-nav-links > ul > li > a span{ color: #9e9e9e }
#ns-top-nav-links > ul > li > a:hover,
#ns-top-nav-links > ul > li > a:focus,
#ns-top-nav-links > ul > li > a:hover span,
#ns-top-nav-links > ul > li > a:focus span,
#ns-top-nav-links a
{ color: #fff }

#ns-top-nav-links > ul > li.active > a,
#ns-top-nav-links > ul > li.active > a span,
#ns-top-nav-links ul.dropdown-menu li.active a
{ color: #e6e6e6 }

#ns-top-nav-links ul.dropdown-menu,
#ns-top-nav-links ul.dropdown-menu li{
    background-color: #4d4d4d;
    color: #fff;
}

#ns-top-nav-links ul.dropdown-menu{
    border-top: 0 none;
}

#ns-top-nav-links ul.dropdown-menu a{ padding: 8px 20px }
#ns-top-nav-links ul.dropdown-menu a:hover{ background-color: #000 }

#ns-top-nav-links ul.dropdown-menu li.active a{ background-color: #222; cursor: default }

body article blockquote{
    background-color: #ffffff;
    border-left: 4px solid red;
}
body article blockquote.style2{ border-left-color: #4195e5 }

.overlay{
    background-color: #000;
    background-color: rgba(0,0,0,0.80);
    color: #fff;
}

.modal-dialog{
    background-color: #222222;
    color: #fff
}

.user-dialog p.text{
    color: #3d3d3d;
}

.inverse{
    background-color: #fff;
    color: #4e4e4e;
}

.inverse a, .inverse a:hover, .inverse a:focus{ color: #4e4e4e }

body article a, .user-dialog a{
    color: #d2482c;
    text-decoration: underline;
    font-weight: bold;
}
body article a:hover,
body article a:focus,
.user-dialog a:hover,
.user-dialog a:focus{
    color: #d2482c;
    text-decoration: none;
}

.user-dialog a.btn{
    color: #fff;
    text-decoration: none
}

#map-toolbar,
#toggle-toolbar,
.map-overlay{
    background: #eaeaea;
    border-left: 1px solid rgba(53, 53, 53, 0.3);
    -webkit-box-shadow: -2px 0px 5px 0px rgba(50, 50, 50, 0.1);
    -moz-box-shadow: -2px 0px 5px 0px rgba(50, 50, 50, 0.1);
    box-shadow: -2px 0px 5px 0px rgba(50, 50, 50, 0.1);
}

#map-toolbar .disabled{
    color: #3d3d3d !important;
    cursor: default;
    background-color: transparent !important;
}
#btn-legend.disabled { cursor: auto }

.btn{ text-decoration: none !important }

.btn,
.input-group-addon,
.has-switch,
.btn.dropdown-toggle .caret{
    text-shadow: 0px -1px 1px rgba(0, 0, 0, 0.5);
}

.btn-success,
.btn-success:hover,
.btn-success:focus,
.btn-success:active{
    background: #3d8113; /* Old browsers */
    background: -moz-linear-gradient(top,  #3d8113 0%, #174f0f 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#3d8113), color-stop(100%,#174f0f)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #3d8113 0%,#174f0f 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #3d8113 0%,#174f0f 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #3d8113 0%,#174f0f 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #3d8113 0%,#174f0f 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#3d8113', endColorstr='#174f0f',GradientType=0 ); /* IE6-9 */
    color: #fff;
    border: 1px solid #367812;
    border-right: 1px solid #16420e;
    border-bottom: 1px solid #16420e;
    outline: 0 none;
    -webkit-box-shadow: inset 1px 1px 1px 0px rgba(124,184,88,1);
    -moz-box-shadow: inset 1px 1px 1px 0px rgba(124,184,88,1);
    box-shadow: inset 1px 1px 1px 0px rgba(124,184,88,1);
}

.btn-success:hover,
.btn-success:focus{
    background: #3e8514;
}

.btn-default,
.btn-default:hover,
.btn-default:focus,
.btn-default:active,
.btn-light,
.input-group-addon,
.btn.dropdown-toggle .caret{
    background: #696b6f; /* Old browsers */
    background: -moz-linear-gradient(top,  #696b6f 0%, #4b4f58 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#696b6f), color-stop(100%,#4b4f58)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #696b6f 0%,#4b4f58 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #696b6f 0%,#4b4f58 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #696b6f 0%,#4b4f58 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #696b6f 0%,#4b4f58 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#696b6f', endColorstr='#4b4f58',GradientType=0 ); /* IE6-9 */
    color: #fff;
    outline: 0 none;
    border-top: 1px solid #747880;
    border-right: 1px solid #40434b ;
    border-bottom: 1px solid #40434b;
    border-left: 1px solid #747880;
    -webkit-box-shadow: inset 1px 1px 1px 0px rgba(166,169,174,1);
    -moz-box-shadow: inset 1px 1px 1px 0px rgba(166,169,174,1);
    box-shadow: inset 1px 1px 1px 0px rgba(166,169,174,1);
}

.btn-default:hover,
.btn-default:focus{
    background: #696b6f;
}

.btn-social,
.btn-social:active{
    background: #5a7dc7; /* Old browsers */
    background: -moz-linear-gradient(top,  #5a7dc7 0%, #4364aa 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#5a7dc7), color-stop(100%,#4364aa)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #5a7dc7 0%,#4364aa 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #5a7dc7 0%,#4364aa 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #5a7dc7 0%,#4364aa 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #5a7dc7 0%,#4364aa 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#5a7dc7', endColorstr='#4364aa',GradientType=0 ); /* IE6-9 */
    border: 1px solid rgba(0,0,0,0.3);
}

.btn-social:hover .hover,
.btn-social:focus .hover{
    display: block;
    background: rgba(255, 255, 255, 0.2)
}

.btn-social span{
    -webkit-box-shadow: inset 3px 3px 4px 0px rgba(255,255,255,0.2);
    -moz-box-shadow: inset 3px 3px 4px 0px rgba(255,255,255,0.2);
    box-shadow: inset 3px 3px 4px 0px rgba(255,255,255,0.2);
}

.btn-social .icon{
    border-right: 1px solid rgba(0,0,0,0.3);
}

.btn-facebook .icon{
    background-color: #3b5998
}

.btn-google,
.btn-google:active{
    background: #d03324; /* Old browsers */
    background: -moz-linear-gradient(top,  #d03324 0%, #df4a32 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#d03324), color-stop(100%,#df4a32)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #d03324 0%,#df4a32 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #d03324 0%,#df4a32 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #d03324 0%,#df4a32 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #d03324 0%,#df4a32 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#d03324', endColorstr='#df4a32',GradientType=0 ); /* IE6-9 */
}

.btn-google .icon{
    background-color: #d03324;
}

.btn-github,
.btn-github:hover{
    background: #212121; /* Old browsers */
    background: -moz-linear-gradient(top,  #212121 0%, #3d3d3d 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#212121), color-stop(100%,#3d3d3d)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #212121 0%,#3d3d3d 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #212121 0%,#3d3d3d 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #212121 0%,#3d3d3d 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #212121 0%,#3d3d3d 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#212121', endColorstr='#3d3d3d',GradientType=0 ); /* IE6-9 */
}

.btn-github .icon{
    background-color: #171516;
}

.input-group-addon, .btn.dropdown-toggle .caret,
.btn-light{
    background: #989aa1; /* Old browsers */
    background: -moz-linear-gradient(top,  #989aa1 0%, #777a81 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#989aa1), color-stop(100%,#777a81)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #989aa1 0%,#777a81 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #989aa1 0%,#777a81 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #989aa1 0%,#777a81 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #989aa1 0%,#777a81 100%) !important; /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#989aa1', endColorstr='#777a81',GradientType=0 ); /* IE6-9 */

    border-top: 1px solid #94969d !important;
    border-right: 1px solid #6d7078 !important;
    border-bottom: 1px solid #52555f !important;
    border-left: 1px solid #6a6d76 !important;
}

.input-group-addon{ padding: 5px 9px !important; }

.input-group-addon:before{
    font-size: 15px !important;
    margin: 0 !important;
    padding: 0 !important;
}

.btn-light:hover,
.btn-light:focus{
    background: #777a81 !important;
    color: #fff !important;
}

.form-control{
    color: #404040;
}

input.error, .error input, .error button, .has-error .form-control{
    border: 1px solid #d96352 !important;
    color: #ff0000 !important;
}
.error .input-group
{
    border: 1px solid #d96352;
}
.error-msg,
.error-list li,
p.has-error{ color: #ff0000 }

#map-legend{
    -webkit-box-shadow: 0px 3px 25px 0px rgba(50, 50, 50, 0.4);
    -moz-box-shadow:    0px 3px 25px 0px rgba(50, 50, 50, 0.4);
    box-shadow:         0px 3px 25px 0px rgba(50, 50, 50, 0.4);
}

#map-legend.inverse a:hover,
#map-legend.inverse a:focus{
    text-decoration: none;
    color: #000;
    outline: 0 none
}

#map-legend.inverse a:hover .text,
#map-legend.inverse a:focus .text{ text-decoration: underline }

#map-legend.inverse .icon-close:hover,
.user-dialog .icon-close:hover,
.inverse .icon-close:focus,
article .icon-close:hover,
article .icon-close:focus{ color: #000 }

#map-legend .stats{ color: #7f7f7f !important; cursor: default }

#map-container .disabled{ opacity: 0.4 }
#map-legend .disabled .text{ text-decoration: none !important }

#map-toolbar a,
#toggle-toolbar{
    border-top: 1px solid #fff;
    border-bottom: 1px solid #d6d6d6;
    color: #3f3f3f;
}

#map-toolbar a:hover,
#map-toolbar a:focus,
#toggle-toolbar:hover,
#toggle-toolbar:focus{
    color: #000;
    background-color: #f3f3f3;
    outline: 0 none
}

#map-toolbar a.active,
#map-toolbar a.active:hover
#map-toolbar a.active:focus{
    border-top-color: #c6c6c4;
    background-color: #e4e4e4;
}

.side-panel{
    background-color: #e4e4e4;
    color: #3f3f3f;
    border: 1px solid #c6c6c4;
    border-right: 0 none;
}

.side-panel a,
.side-panel a:active{
    color: #3f3f3f;
}
.side-panel a:hover,
.side-panel a:focus{
    color: #000;
}

.side-panel .tool{
    background-color: transparent;
    border: 1px solid #e4e4e4;
    color: #5f5e63;
}

.side-panel .tool:hover,
.side-panel .tool:focus{
    border: 1px solid #9e9e9e;
    border-top-color: #fefefe;
    border-left-color: #fefefe;
    background: #ebebeb; /* Old browsers */
    background: -moz-linear-gradient(top,  #ebebeb 0%, #d4d4d4 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#ebebeb), color-stop(100%,#d4d4d4)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #ebebeb 0%,#d4d4d4 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #ebebeb 0%,#d4d4d4 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #ebebeb 0%,#d4d4d4 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #ebebeb 0%,#d4d4d4 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#ebebeb', endColorstr='#d4d4d4',GradientType=0 ); /* IE6-9 */
    outline: 0 none;
}

.side-panel .tool.active,
.side-panel .tool:active,
.side-panel .tool.active:hover,
.side-panel .tool.active:focus{
    border: 1px solid #d9d9d9;
    border-top: 1px solid #8d8d8d;
    border-left: 1px solid #8d8d8d;
    background: #ffffff; /* Old browsers */
    background: -moz-linear-gradient(top,  #ffffff 0%, #f6f6f4 100%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#ffffff), color-stop(100%,#f6f6f4)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #ffffff 0%,#f6f6f4 100%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #ffffff 0%,#f6f6f4 100%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #ffffff 0%,#f6f6f4 100%); /* IE10+ */
    background: linear-gradient(to bottom,  #ffffff 0%,#f6f6f4 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#ffffff', endColorstr='#f6f6f4',GradientType=0 ); /* IE6-9 */
}

.loading .icon-close{
    color: #fff;
    opacity: 0.4;
}
.loading .icon-close:hover{
    opacity: 0.9;
    text-decoration: none
}

.user-dialog .icon-close,
article .icon-close{
    color: #999999;
}

#preloader{
    background: #fff;
    color: #333;
}

.has-switch{ width: 76px; min-width: 76px !important }
.has-switch span{
    font-size: 12px;
    border-radius: 3px !important;
    padding-left: 6px;
    padding-right: 6px;
    width: 37px;
    text-align: center;
}

.has-switch span.switch-primary,
.has-switch span.switch-left,
input[type="range"]{
    border-top: 1px solid #747880;
    border-right: 1px solid #62656e;
    border-bottom: 1px solid #63666e;
    border-left: 1px solid #61656d;
    border-radius: 3px !important;
    background: #989aa1; /* Old browsers */
    background: -moz-linear-gradient(top,  #989aa1 0%, #777a81 86%); /* FF3.6+ */
    background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#989aa1), color-stop(86%,#777a81)); /* Chrome,Safari4+ */
    background: -webkit-linear-gradient(top,  #989aa1 0%,#777a81 86%); /* Chrome10+,Safari5.1+ */
    background: -o-linear-gradient(top,  #989aa1 0%,#777a81 86%); /* Opera 11.10+ */
    background: -ms-linear-gradient(top,  #989aa1 0%,#777a81 86%); /* IE10+ */
    background: linear-gradient(to bottom,  #989aa1 0%,#777a81 86%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#989aa1', endColorstr='#777a81',GradientType=0 ); /* IE6-9 */
    -webkit-box-shadow: inset 0px 1px 1px 0px rgba(166,169,174,1);
    -moz-box-shadow: inset 0px 1px 1px 0px rgba(166,169,174,1);
    box-shadow: inset 0px 1px 1px 0px rgba(166,169,174,1);
}

.has-switch span.switch-default,
.has-switch span.switch-right{
    color: #7c868d;
    text-shadow: -1px -1px 1px rgba(0, 0, 0, 0.2);
    border-top: 1px solid #c1c1c1;
    border-right: 1px solid #828282;
    border-bottom: 1px solid #828282;
    border-left: 1px solid #c1c1c1;
    border-radius: 3px !important;
}

.has-switch:focus{
    border-color: rgba(0, 0, 0, 0.3);
    -webkit-box-shadow: inset 0 1px 1px rgba(0,0,0,.075), 0 0 8px rgba(172,179,192,1);
    box-shadow: inset 0 1px 1px rgba(0,0,0,.075), 0 0 8px rgba(172,179,192,1);
}

.custom-checkbox {
    display: inline-block;
    position: relative;
    width: 18px;
    height: 18px;
    vertical-align: top;
    margin-right: 5px;
}

.custom-checkbox input{
    display: none;
}

.custom-checkbox label {
    cursor: pointer;
    position: absolute;
    width: 18px;
    height: 18px;
    top: 0;
    border-radius: 4px;
    border: 1px solid #cfcfcf;

    -webkit-box-shadow: inset 0px 1px 1px white, 0px 1px 2px rgba(0,0,0,0.2);
    -moz-box-shadow: inset 0px 1px 1px white, 0px 1px 2px rgba(0,0,0,0.2);
    box-shadow: inset 0px 1px 1px white, 0px 1px 2px rgba(0,0,0,0.2);
    background: #fcfff4;
}

.custom-checkbox label:after {
    -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=0)";
    filter: alpha(opacity=0);
    opacity: 0;
    content: '';
    position: absolute;
    width: 11px;
    height: 7px;
    background: transparent;
    top: 3px;
    left: 3px;
    border: 3px solid #4d515b;
    border-top: none;
    border-right: none;

    -webkit-transform: rotate(-45deg);
    -moz-transform: rotate(-45deg);
    -o-transform: rotate(-45deg);
    -ms-transform: rotate(-45deg);
    transform: rotate(-45deg);
}

.custom-checkbox input[type=checkbox]:checked + label:after {
    -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=100)";
    filter: alpha(opacity=100);
    opacity: 1;
}

.custom-radio {
    display: inline-block;
    position: relative;
    width: 18px;
    height: 18px;
    vertical-align: top;
    margin-right: 5px;
}

.custom-radio input{
    display: none;
}

.custom-radio label {
    cursor: pointer;
    position: absolute;
    width: 18px;
    height: 18px;
    top: 0;
    border-radius: 50%;
    border: 1px solid #cfcfcf;

    -webkit-box-shadow: inset 0px 1px 1px white, 0px 1px 2px rgba(0,0,0,0.2);
    -moz-box-shadow: inset 0px 1px 1px white, 0px 1px 2px rgba(0,0,0,0.2);
    box-shadow: inset 0px 1px 1px white, 0px 1px 2px rgba(0,0,0,0.2);
    background: #fcfff4;
}

.custom-radio label:after {
    -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=0)";
    filter: alpha(opacity=0);
    opacity: 0;
    content: '';
    position: absolute;
    width: 8px;
    height: 8px;
    background: #4d515b;
    top: 4px;
    left: 4px;
    border-radius: 50%;

    -webkit-transform: rotate(-45deg);
    -moz-transform: rotate(-45deg);
    -o-transform: rotate(-45deg);
    -ms-transform: rotate(-45deg);
    transform: rotate(-45deg);
}

.custom-radio input[type=radio]:checked + label:after {
    -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=100)";
    filter: alpha(opacity=100);
    opacity: 1;
}

.slider {
    display: inline-block;
    width: 100%;
    padding-top: 6px;
}

input[type="range"] {
    -webkit-appearance: none !important;
    width: 100%;
    height: 9px;
    border: 1px solid #5b6066;
    border-radius: 10px !important;
    margin: auto;
    transition: all 0.3s ease;
}
input[type="range"]:hover {
    background-color: #b2d5aa;
}

input[type="range"]::-webkit-slider-thumb {
    -webkit-appearance: none !important;
    width: 20px;
    height: 20px;
    background-color: #f9f9f9;
    border: 1px solid #d0d0d0;
    border-radius: 50%;
    box-shadow: 0px 0px 3px rgba(0,0,0,0.4);
    transition: all 0.5s ease;
    cursor: pointer;
}
input[type="range"]::-webkit-slider-thumb:hover {
    background-color: #fff;
}
input[type="range"]::-webkit-slider-thumb:active {
    box-shadow: 0px 0px 1px #3c6d59;
}

#rangevalue {
    text-align: center;
    font-family: "Arial", sans-serif;
    font-size: 12px;
    display: block;
    margin: auto;
    padding: 10px 0px 0;
    width: 100%;
    font-weight: bold;
}

.btn-group.dropdown,
.bootstrap-select{
    height: 28px;
    margin-bottom: 10px;
}

.btn-group.dropdown,
.btn.dropdown-toggle,
.bootstrap-select{
    display: block !important;
    width: 100%;
    padding: 0;
}

.bootstrap-select{
    width: 100% !important;
}

.bootstrap-select.btn-group .btn .caret{
    top: 0;
    right: 0;
    margin: 0;
}

.btn.dropdown-toggle{
    position: relative;
    padding: 4px 9px;
    border: 1px solid #cccccc;
    background-color: #ffffff;
    color: #404040;
    text-shadow: none;
    text-align: left;
    font-size: 13px;
}

.btn.dropdown-toggle .caret{
    position: absolute;
    top: 0;
    right: 0;
    border: 0 none;
    width: 28px;
    height: 27px;
    padding: 0 !important;
    text-align: center;
    border-radius: 0px 3px 3px 0px;
}

.btn.dropdown-toggle .caret:before{
    content: '\25BC';
    display: block;
    font-size: 13px !important;
    line-height: 28px;
}
#body .input-sm .caret:before,
#map-panels .caret:before,
#node-list footer .caret:before{
    font-size: 12px !important;
    line-height: 22px !important;
}

/* TODO: probably unused */
.btn.dropdown-toggle .icon-down{
    font-size: 17px;
    position: relative;
    top: -2px;
    right: -2px;
}

.btn-group.bootstrap-select .bootstrap-select-searchbox input{
    height: 28px !important
}

.btn-group.bootstrap-select ul.dropdown-menu{
    width: 100%;
    margin-top: -1px;
}

.btn-group.bootstrap-select ul.dropdown-menu a{
    text-decoration: none;
    padding: 8px 9px;
    font-size: 13px;
}

label{ font-weight: normal !important }
label.inline{ margin: 0 }

.line{
    margin-bottom: 10px;
    border-top: 1px solid #fff;
    border-bottom: 1px solid #d2d2d2
}

.separator .text{
    background-color: #fff;
    color: #7d7d7d;
}

.panel-inner{
    background-color: #f1f1f1;
    border: 1px solid #dadada;
}

#js-password-strength-message .progress{
    border: 1px solid #ccc;
}

.cluster{
    width: 37px !important;
    height: 37px !important;
    background-color: blue;
    color: #fff;
    text-align: center;
    font: bold 13px/37px Arial, Helvetica, sans-serif;
    border-radius: 50%;
    box-sizing: content-box;
}

.cluster-size-0{
    width: 24px !important;
    height: 24px !important;
    line-height: 25px !important;
}

.cluster-size-1{
    width: 33px !important;
    height: 33px !important;
    line-height: 33px !important;
}

.cluster-size-2{
    width: 45px !important;
    height: 45px !important;
    line-height: 45px !important;
}

.cluster-size-3{
    width: 55px !important;
    height: 55px !important;
    line-height: 55px !important;
}

.leaflet-cluster-anim .leaflet-marker-icon, .leaflet-cluster-anim .leaflet-marker-shadow {
    -webkit-transition: -webkit-transform 0.3s ease-out, opacity 0.3s ease-in;
    -moz-transition: -moz-transform 0.3s ease-out, opacity 0.3s ease-in;
    -o-transition: -o-transform 0.3s ease-out, opacity 0.3s ease-in;
    transition: transform 0.3s ease-out, opacity 0.3s ease-in;
}

.icon-ok,
.icon-thumbs-up{ color: #328327 !important }
.icon-cancel,
.icon-thumbs-down{ color: #c72c25 !important }
.vote .fade{ opacity: 0.2 }
.vote .disabled{ opacity: 1 !important; cursor: default }

a.icon-thumbs-up:hover{ color: #139502 }
a.icon-thumbs-down:hover{ color: #e20a00 }

.icon-ok,
.icon-cancel,
.icon-star-full,
.icon-star-half,
.icon-star-empty,
.icon-thumbs-up,
.icon-thumbs-down{
    line-height: 0;
    font-size: 18px;
    position: relative;
    bottom: -1px;
}

.status{
    padding: 2px 9px;
    background-color: #595959;
    color: #fff;
    border-radius: 5px;
}
.status.pull-left{
    margin-top: 5px;
    margin-left: 2.4%;
    padding-bottom: 3px;
}

.icon-star-full,
.icon-star-half,
.icon-star-empty{
    color: #ffa803;
}

.icon-star-full:before,
.icon-star-half:before,
.icon-star-empty:before,
.icon-thumbs-up:before,
.icon-thumbs-down:before{
    width: auto !important;
    margin: 0 1px 0 0 !important;
}

a.rating,
a.icon-thumbs-up,
a.icon-thumbs-down{
    text-decoration: none;
}

.icon-thumbs-up,
.icon-thumbs-down{
    margin-left: 7px;
}

.icon-thumbs-up:before,
.icon-thumbs-down:before{
    font-size: 30px;
}

.icon-thumbs-down:before{
    position: relative;
    bottom: -6px
}

.details-background, .short-map{
    background-color: #aba49c;
}

#map .leaflet-draw-tooltip{
    padding: 10px 15px;
    font-size: 14px;
    line-height: 25px;
}

#map .leaflet-draw-tooltip .btn{
    margin-top: 5px;
    color: #fff
}
